package com.mortals.xhx.module.record.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.common.code.CertificateStatus;
import com.mortals.xhx.common.code.ProcessStatusEnum;
import com.mortals.xhx.common.code.StatusEnum;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.certificate.model.CertificateDocumentEntity;
import com.mortals.xhx.module.certificate.model.vo.CertificateDocumentPdu;
import com.mortals.xhx.module.child.model.ChildLicenseEntity;
import com.mortals.xhx.module.child.model.ChildLicenseQuery;
import com.mortals.xhx.module.child.service.ChildLicenseService;
import com.mortals.xhx.module.record.dao.ApplyLogDao;
import com.mortals.xhx.module.record.dao.RetainLogDao;
import com.mortals.xhx.module.record.model.ApplyLogEntity;
import com.mortals.xhx.module.record.model.RetainLogEntity;
import com.mortals.xhx.module.record.model.RetainLogQuery;
import com.mortals.xhx.module.record.service.ApplyLogService;
import com.mortals.xhx.module.record.service.RetainLogService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
* RetainLogService
* 证照持有 service实现
*
* @author zxfei
* @date 2022-10-14
*/
@Service("retainLogService")
public class RetainLogServiceImpl extends AbstractCRUDServiceImpl<RetainLogDao, RetainLogEntity, Long> implements RetainLogService {

    @Autowired
    private ChildLicenseService childLicenseService;
    @Autowired
    private ApplyLogDao applyLogDao;

    @Override
    protected RetainLogEntity findBefore(RetainLogEntity entity, PageInfo pageInfo, Context context) throws AppException {
        RetainLogQuery query = new RetainLogQuery();
        BeanUtils.copyProperties(entity,query);
        if(StringUtils.isNotEmpty(entity.getQuery())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(entity.getQuery()).append("%");
            RetainLogQuery condition1 = new RetainLogQuery();
            RetainLogQuery condition2 = new RetainLogQuery();
            condition1.setHolderName(condition.toString());
            condition2.setCertificateCode(condition.toString());
            List<RetainLogQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            query.setOrConditionList(orConditionList);
        }
        return query;
    }

    @Override
    protected void findAfter(RetainLogEntity params, PageInfo pageInfo, Context context, List<RetainLogEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            for(RetainLogEntity entity:list){
                List<ChildLicenseEntity> childLicenseList = childLicenseService.find(new ChildLicenseQuery().applyId(entity.getApplyId()));
//                if(CollectionUtils.isEmpty(childLicenseList)){
//                    entity.setProcessStatus("0/0");
//                }else {
//                    Map<Integer, List<ChildLicenseEntity>> group = childLicenseList.stream().collect(Collectors.groupingBy(x -> x.getProcessStatus()));
//                    int pCount = CollectionUtils.isEmpty(group.get(1)) ? 0 : group.get(1).size();
//                    entity.setProcessStatus(pCount + "/" + childLicenseList.size());
//                }
                int childSum = 0;
                int pCount = 0;
                List<CertificateDocumentPdu> documentEntityList = new ArrayList<>();
                for(ChildLicenseEntity item:childLicenseList){
                    CertificateDocumentEntity documentEntity = new CertificateDocumentEntity();
                    BeanUtils.copyProperties(item,documentEntity);
                    documentEntity.setId(item.getDocumentId());
                    documentEntity.setStatus(StatusEnum.DISABLE.getValue());
                    if(item.getChildStatus()==StatusEnum.ENABLE.getValue()){
                        documentEntity.setStatus(StatusEnum.ENABLE.getValue());
                        childSum++;
                        if(item.getProcessStatus()== ProcessStatusEnum.已处理.getValue().intValue()){
                            pCount++;
                        }
                    }
                    CertificateDocumentPdu pdu = new CertificateDocumentPdu();
                    BeanUtils.copyProperties(documentEntity, pdu);
                    documentEntityList.add(pdu);
                }
                entity.setProcessStatus(pCount + "/" + childSum);
            }
        }
    }

    @Override
    protected void updateAfter(RetainLogEntity entity, Context context) throws AppException {
        if(entity.getCertificateStatus()== CertificateStatus.CANCEL.getValue()){
            ApplyLogEntity applyLogEntity = applyLogDao.get(entity.getApplyId());
            if(applyLogEntity!=null){
                ApplyLogEntity update = new ApplyLogEntity();
                update.setId(applyLogEntity.getId());
                update.setSocialCode(applyLogEntity.getSocialCode()+"_zx");
                update.setUpdateTime(new Date());
                applyLogDao.update(update);
                List<ChildLicenseEntity> childLicenseEntityList = childLicenseService.find(new ChildLicenseQuery().applyId(applyLogEntity.getId()));
                childLicenseService.removeList(childLicenseEntityList,context);
            }
        }
    }
}